/*
 *
 *  Created on: Mar 1, 2021
 *      Author: astrisw
 */
#pragma once

#include <Base_Protocol.h>
#include <Base_Configurator.h>

/*
 * class TCPSocket
 * Implementation of a TCP socket
 * @member field fd_sock: socket file descriptor
 */
namespace inaf::oasbo::ConnectionProtocols {

class TCPProtocol: public BaseProtocol {
protected:
	TCPProtocol(std::string ip, int prt);
	TCPProtocol();
	std::string ip;
	int port;
	int srv_sock;
	int cli_sock;
	int m_connectToSrv(std::string ip, int port);
	int m_connectToCli(std::string ip, int port);
	void resetPacket(inaf::oasbo::PacketLib::BasePacket &pack, int bytes);
	bool split_ip_port(const std::string &ip_port, std::string &ip,
			std::string &port);


public:
	~TCPProtocol();

	std::string getHost() override;
	void setHost(std::string host) override;

	void setIp(std::string);
	std::string getIp(){return this->ip;}
	void setPort(int);
	int getPort(){return this->port;}

	int connectToServer() override;
	int connectToClient() override;
	int closeConnectionToServer() override;
	int closeConnectionToClient() override;
	bool isConnectedToClient() const override;
	bool isConnectedToServer() const override;
	int receiveFromClient(PacketLib::BasePacket&) override;
	int sendToServer(PacketLib::BasePacket&) override;

	friend class TCPProtocolBuilder;
};
class TCPProtocolBuilder {
protected:
	TCPProtocol *protocol;

public:
	std::string config_target { "tcpprotocol" };
	std::string ip_key { "ip" };
	std::string port_key { "port" };

	TCPProtocolBuilder();
	TCPProtocolBuilder(std::string ip, int port);
	~TCPProtocolBuilder();
	void reset();
	TCPProtocolBuilder* configFrom(Configurators::BaseConfigurator *conf);
	TCPProtocolBuilder* setIp(std::string ip);
	TCPProtocolBuilder* setPort(int port);
	TCPProtocol* getProtocol();
};
}
